@@ -1,7 +1,7 @@
 /* 
 Copyright (C) 2011  Fabian Kurz, DJ1YFK
  
-$Id: pulseaudio.c 388 2011-12-18 21:28:50Z dj1yfk $
+$Id: pulseaudio.c 541 2012-08-17 17:58:26Z dj1yfk $
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -42,11 +42,12 @@ void *open_dsp () {
 	}
 
 	/* The Sample format to use */
-	static const pa_sample_spec ss = {
+	static pa_sample_spec ss = {
 		.format = PA_SAMPLE_S16LE,
-		.rate = 44100,
+		.rate = 8000,
 		.channels = 1
 	};
+	ss.rate = samplerate;
 	pa_simple *s = NULL;
 	int error;
 
