@@ -1,8 +1,8 @@
 /* 
 qrq - High speed morse trainer, similar to the DOS classic "Rufz"
-Copyright (C) 2006-2011  Fabian Kurz
+Copyright (C) 2006-2013  Fabian Kurz
 
-$Id: qrq.c 388 2011-12-18 21:28:50Z dj1yfk $
+$Id: qrq.c 564 2013-01-06 13:25:46Z dj1yfk $
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -190,7 +190,9 @@ int main (int argc, char *argv[]) {
 	char abort = 0;
 	char tmp[80]="";
 	char input[15]="";
-	int i=0,j=0;						/* counter etc. */
+	int i=0,j=0,k=0;						/* counter etc. */
+	char previouscall[80]="";
+	int previousfreq = 0;
 	int f6pressed=0;
 
 	if (argc > 1) {
@@ -204,7 +206,7 @@ int main (int argc, char *argv[]) {
 	keypad(stdscr, TRUE);
 	scrollok(stdscr, FALSE);
 	
-	printw("qrq v%s - Copyright (C) 2006-2011 Fabian Kurz, DJ1YFK\n", VERSION);
+	printw("qrq v%s - Copyright (C) 2006-2013 Fabian Kurz, DJ1YFK\n", VERSION);
 	printw("This is free software, and you are welcome to redistribute it\n");
 	printw("under certain conditions (see COPYING).\n");
 
@@ -440,11 +442,10 @@ while (status == 1) {
 		
 		f6pressed=0;
 
-	
 		while (!abort && (j = readline(bot_w, 1, 8, input,1)) > 4) {/* F5..F10 pressed */
 
 			switch (j) {
-				case 6:
+				case 6:		/* repeat call */
 				if (f6pressed && (f6 == 0)) {
 					continue;
 				}
@@ -460,6 +461,26 @@ while (status == 1) {
 			thread_fail(j);
 #endif	
 					break; /* 6*/
+				case 7:		/* repeat _previous_ call */
+					if (callnr > 1) {
+						k = freq;
+						freq = previousfreq;
+#ifdef WIN_THREADS
+			WaitForSingleObject(cwthread,INFINITE);
+			cwthread = (HANDLE) _beginthread( morse,0,previouscall);
+			WaitForSingleObject(cwthread,INFINITE);
+#else
+			pthread_join(cwthread, NULL);
+			j = pthread_create(&cwthread, NULL, &morse, previouscall);	
+			thread_fail(j);
+			pthread_join(cwthread, NULL);
+#endif	
+						/* NB: We must wait for the CW thread before
+						 * we set the freq back -- this blocks keyboard
+						 * input, but in this case it shouldn't matter */
+						freq = k;
+					}
+					break;
 				case 10:	/* abort attempt */
 					abort = 1;
 					continue;
@@ -482,6 +503,8 @@ while (status == 1) {
 				show_error(calls[i], tmp);
 		}
 		input[0]='\0';
+		strncpy(previouscall, calls[i], 80);
+		previousfreq = freq;
 		calls[i] = NULL;
 	}
 
@@ -1069,8 +1092,8 @@ static int add_to_toplist(char * mycall, int score, int maxspeed) {
 	timestamp = (int) time(NULL);
 	sprintf(insertline, "%-10s%6d %3d %10d", mycall, score, maxspeed, timestamp);
 	
-	if ((fh = fopen("toplist", "rb+")) == NULL) {
-		printf("Unable to open toplist file 'toplist'!\n");
+	if ((fh = fopen(tlfilename, "rb+")) == NULL) {
+		printf("Unable to open toplist file %s!\n", tlfilename);
 		exit(EXIT_FAILURE);
 	}
 
@@ -1292,6 +1315,14 @@ static int read_config () {
 								"Using default >%s<.\n", line, tmp, cbfilename);
 			}
 		}
+		else if (tmp == strstr(tmp,"samplerate=")) {
+			while (isdigit(tmp[i] = tmp[11+i])) {
+				i++;
+			}
+			tmp[i]='\0';
+			samplerate = atoi(tmp);
+			printw("  line  %2d: sample rate: %d\n", line, samplerate);
+		}
 	}
 
 	printw("Finished reading qrqrc.\n");
@@ -1330,7 +1361,7 @@ static void *morse(void *arg) {
 	full_bufpos = 0; 
 
 	/* Some silence; otherwise the call starts right after pressing enter */
-	tonegen(0, 11025, SILENCE);
+	tonegen(0, samplerate/4, SILENCE);
 
 	/* Farnsworth? */
 	if (speed < mincharspeed) {
@@ -2059,7 +2090,7 @@ void select_callbase () {
 
 
 void help () {
-		printf("qrq v%s  (c) 2006-2011 Fabian Kurz, DJ1YFK. "
+		printf("qrq v%s  (c) 2006-2013 Fabian Kurz, DJ1YFK. "
 					"http://fkurz.net/ham/qrq.html\n", VERSION);
 		printf("High speed morse telegraphy trainer, similar to"
 					" RUFZ.\n\n");
